/*
Project Name     : DTMF Dialer with Holtek HT9200A (Serial Mode) v1.0
Project Author   : Petar Timotijevic ptimotijevic@gmail.com
Published        : www.libstock.com
Compiler         : MikroC Pro for PIC 6.0.1
MCU              : PIC45K22  @ 32MHz (8MHz x 4xPLL)
Development Sys  : EasyPIC v7
Pins Description : PortC (RC1) to HT9200A CE   Pin 1 (Chip On is low, Chip Off is high)
                   PortC (RC4) to HT9200A DATA Pin 6 (D0,D1,D2,D3 bits)
                   PortC (RC3) to HT9200A CLK  Pin 5 (Data synchronous clock)
                                  HT9200A DTMF Pin 7 (DTMF Signal Output 0,45V-0,75V)
Description      : HT9200A uses a 5-bit code, and a synchronous clock to generate a DTMF signal.
                   Each DTMF digit is made by serie of 5 bits. D0 is LSB (first received bit).
                   HT9200A will latch data on the falling edge of the clock (CLK pin).
*/

//HT9200A DTMF CODING TABLE (5bits for each DTMF code)
char d_1[]  = "10000";   // 1
char d_2[]  = "01000";   // 2
char d_3[]  = "11000";   // 3
char d_4[]  = "00100";   // 4
char d_5[]  = "10100";   // 5
char d_6[]  = "01100";   // 6
char d_7[]  = "11100";   // 7
char d_8[]  = "00010";   // 8
char d_9[]  = "10010";   // 9
char d_0[]  = "01010";   // 0
char d_G[]  = "11010";   // *
char d_F[]  = "00110";   // #
char d_AA[] = "10110";   // A
char d_BB[] = "01110";   // B
char d_CC[] = "11110";   // C
char d_DD[] = "00000";   // D
char d_EN[] = "11111";   // DTMF Sequence END

const DigitPause  = 150;     //  <<----- Here define pause between DTMF codes in ms ( >= 1)
const DigitDurat  = 400;     //  <<----- Here define digit of each digit sound in ms ( >= 1)

char  DTMF_Code[20] = "";
char i, ii, lng[5], digit;
int num;


void SendBits(char DigitBits[5])
     {
     for(i=0;i<=4;i++)            // Loop for 5 bits for each digit
         {
         Delay_us(5);             // Wait 5us
         PortC.F3 = 1;            // CLK Enable
         PortC.F4 = DigitBits[i]; // Take data bit
         Delay_us(5);             // Wait 5us
         PortC.F3 = 0;            // CLK Disable
         Delay_us(5);             // Wait 5us
         }
     }


void StopBits()  //// SEND 11111 ON SEQUENCE END
     {
     SendBits(d_EN);
     }


void DTMF(char DIN[])
     {
     strcpy(DTMF_code, DIN); // Copy input DTMF code sequence from DIN to DTMF_Code

// DTMF("1234567890");       // <<-- Here define DTMF code sequence

   ANSELC = 0;               // Configure PORTC pins as digital
   TRISC  = 0;               // Configure PORTC pins as output

   num = strlen(DTMF_Code);  //Calculates the length of the DTMF_Code
   shorttostr(num, lng);


// HT9200A INITIALIZATION - Respect five code lines of starting sequence bellow
   PortC.F1 = 1;   // CE Turn Off chip, this is very important
   PortC.F3 = 0;   // Power Up with CLK Disabled, this is very important
   PortC.F3 = 1;   // Set CLK high before startup, this is very important
   PortC.F1 = 0;   // CE turn chip ON
   Delay_ms(10);   // Wait 10ms for oscillator stabilization


 for(ii=0;ii<=num-1;ii++)  // Loop for each DTMF code
       {
       digit = DTMF_Code[ii];

       switch (digit)
           {
           case '1': { 
                     SendBits(d_1);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case '2': {
                     SendBits(d_2);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case '3': {
                     SendBits(d_3);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case '4': {
                     SendBits(d_4);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case '5': {
                     SendBits(d_5);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case '6': {
                     SendBits(d_6);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case '7': {
                     SendBits(d_7);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case '8': {
                     SendBits(d_8);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case '9': {
                     SendBits(d_9);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case '0': {
                     SendBits(d_0);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case '*': {
                     SendBits(d_G);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case '#': {
                     SendBits(d_F);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case 'A': {
                     SendBits(d_AA);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case 'B': {
                     SendBits(d_BB);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case 'C': {
                     SendBits(d_CC);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }
           case 'D': {
                     SendBits(d_DD);
                     Delay_ms(DigitDurat);    // Sound duration of digit
                     StopBits();              // Send stop bits after digit, this allows pause between digits
                     break;
                     }

           Delay_ms(DigitPause);              // Here is pause between each digit
           }

       }
   
}